<?php
/*
Plugin Name: Peter's Custom Anti-Spam
Plugin URI: http://www.theblog.ca/?p=21
Description: Stop a lot of spambots from polluting your site by making visitors identify a random word displayed as an image before commenting. You can customize the pool of words to display. This was originally based on <a href="http://www.infor96.com/~nio/archives/369">Anti-Spam Image</a> by Krazy Nio. Core of the Version 2.0 rewrite by <a href="http://WillMurray.name/">Will Murray</a>.
Author: Peter Keung
Version: 2.1
Author URI: http://www.theblog.ca/
Change Log:
2006-10-14  Version 2.1. Increased default width of image and allowed customization of border color.
2006-09-15  Version 2.0. Core rewrite by Will Murray (http://WillMurray.name/):
	* Added ability to use multiple, random fonts.
	* Added random text color selection.
	* Added limited ability to pick background color (white or black) for better compatibility with light or dark themes.
	* Added custom (not yet dynamic) image sizes.
	* Added the option to output PNG (clearer text, but may break older browsers -- your choice)
	* Added additional input validation for improved security.
	* Improved speed and efficiency by adding conditionals and reducing redundant code.
	* Added "cas_" prefix to custom_anti_spam variables to avoid conflicts with other plugins.
	* Added self-checking of plugin location (placing the plugin inside a subfolder of wp-content/plugins is now suggested).
	* Added ability to forgo basic JavaScript positioning and insert the file yourself (Thanks to TerminalDigit)
View all previous change items at http://www.theblog.ca/?page_id=50
*/

// -----------------------------------------------------------------------------------
// Customize these parts if you want to change the defaults.

// Set this to equal TRUE if you want to force registered users to enter the anti-spam word as well.
$cas_forcereg = false;

// Set this to equal TRUE if you want to allow trackbacks (but be vulnerable to trackback spam)
$cas_allowtrack = false;

$cas_text = array(); // <-- This line should not be changed
// List as many words as you like, one per line
// If you want some words to be used more often, enter them multiple times.
// It is best to use words that are seven letters or less so that they don't get cut off (otherwise, increase the default image size a bit further down the code)
$cas_text[] = "comet";
$cas_text[] = "natural";
$cas_text[] = "grape";
$cas_text[] = "marker";
$cas_text[] = "jazz";
$cas_text[] = "blake";
$cas_text[] = "kitten";
$cas_text[] = "swish";
$cas_text[] = "listen";
$cas_text[] = "island";

// Set the path to the font(s). Example: "wp-content/fonts/"
// Empty string ("") defaults to the root of your blog.
// If you add a path, DO include the trailing slash.
$cas_fontpath = "wp-content/plugins/custom-anti-spam/";

$cas_fontlist = array(); // <-- This line should not be changed
// List as many TrueType font(s) as you like, one per line. Drop your own font files into this plugin's directory.
// If you are using your own fonts, make sure all fonts used are about the same default size.
// If you want some fonts to be used more frequently, enter them multiple times.
// Default freeware fonts from font101.com
$cas_fontlist[] = "jenkinsv.ttf";
$cas_fontlist[] = "beatty.ttf";
$cas_fontlist[] = "ateliersans.ttf";

// Set the anti-spam image width and height.
// You may need to increase these sizes for longer words and/or bigger fonts.
$cas_imgwidth = 160;
$cas_imgheight = 50;

// Set this to TRUE if you want to use random text colors.
// If random colors are not selected, blue text will appear on a white background, and white text will appear on black background (as decided in the next option)
$cas_randomcolors = true;

// Set the background color for the anti-spam image.
// Choose either "black" or "white"
$cas_bgcolorset = "white";

// Set the border color for the anti-spam image.
// Write either major colors (red, green, blue, etc.) or enter the HTML color code (such as #C0C0C0)
$cas_borderclr = "black";

// Set this to TRUE if you prefer PNG graphics (better quality text)
// Set this to FALSE if you prefer more compatable graphics (PNG crashes IE 4; JPEG does not)
$cas_UsePngNotJpeg = false;

// Set this to TRUE if you will be editing your comments file (add this php line wherever you want the anti-spam image inserted in the comments.php file: do_action('secure_image', $post->ID); )
// Set this to FALSE if you want to use the default Javascript positioning
$cas_manualinsert = false;

// -----------------------------------------------------------------------------------
// You should not need to (and probably shouldn't) edit anything from here to the end.

// Determine the ABSPATH if it is not already defined (i.e., when called to generate the image)
if( ! defined( 'ABSPATH' ) )
{
	$abspath = dirname( __FILE__ ) . '/';
	$absarray = explode( "/", $abspath );
	$abspath = "";
	foreach( $absarray as $value )
	{
		$abspath = $abspath . $value . "/";
		if( file_exists( $abspath . "wp-config.php" ) )
		{
			if( ! defined( 'ABSPATH' ) ) define( 'ABSPATH', $abspath, true );
		}
	}
}
if( ! defined( 'ABSPATH' ) ) die( "Could not determine the ABSPATH (the root of the blog)." );
$cas_fontpath = ABSPATH . $cas_fontpath;

// Determine how many words were entered
$cas_textcount = count( $cas_text );
// Copy the first element to a new last element
$cas_text[] = $cas_text[0];
// Set the first element to invalid
$cas_text[0] = "* * * INVALID * * *";

// Output the antispam image
if( isset( $_GET['antiselect'] ) )
{
	// Pick a random font to use
	$cas_font = $cas_fontpath . $cas_fontlist[ rand( 0, count( $cas_fontlist ) - 1 ) ];

	// Set the default colors for when random text colors are not selected
	if ( $cas_bgcolorset == "white") {
	$cas_textcolor = array( 0, 0, 255 ); // blue text
	$cas_bgcolor = array( 255, 255, 255); // white background
	}
	else {
	$cas_textcolor = array( 255, 255, 255 ); // white text
	$cas_bgcolor = array( 0, 0, 0); // black background
	}

	// If selected, pick a random color for the antispam word text
	if( $cas_randomcolors )
	{
		$cas_rand = rand( 0, 4 );
		switch( $cas_bgcolorset )
			{
				case "white":
					$cas_textcolorchoice[0] = array ( 0, 0, 255 ); // blue
					$cas_textcolorchoice[1] = array ( 0, 153, 0 ); // greenish
					$cas_textcolorchoice[2] = array ( 204, 0, 0 ); // reddish
					$cas_textcolorchoice[3] = array ( 203, 0, 154 ); // purplish
					$cas_textcolorchoice[4] = array ( 0, 0, 0 ); // black
					$cas_textcolor = $cas_textcolorchoice[$cas_rand];
					break;
				default:
					$cas_textcolorchoice[0] = array ( 255, 255, 0 ); // yellow
					$cas_textcolorchoice[1] = array ( 0, 255, 255 ); // blueish
					$cas_textcolorchoice[2] = array ( 255, 153, 204 ); // pinkish
					$cas_textcolorchoice[3] = array ( 102, 255, 102 ); // greenish
					$cas_textcolorchoice[4] = array ( 255, 255, 255 ); // white
					$cas_textcolor = $cas_textcolorchoice[$cas_rand];
					break;
			}
	}

	// Validate the input values
	$cas_antiselect = intval( $_GET['antiselect'] );
	if( $cas_antiselect < 1 || $cas_antiselect > $cas_textcount ) $cas_antiselect = 0;
	$cas_antispam = $cas_text[ $cas_antiselect ];

	// Start building the image
	$cas_image = @imagecreate( $cas_imgwidth, $cas_imgheight ) or die("Cannot Initialize new GD image stream");
	$cas_bgcolor = imagecolorallocate( $cas_image, $cas_bgcolor[0], $cas_bgcolor[1], $cas_bgcolor[2] );
	$cas_fontcolor = imagecolorallocate( $cas_image, $cas_textcolor[0], $cas_textcolor[1], $cas_textcolor[2] );

	// Check for freetype lib, if not found default to ugly built in capability using imagechar (Lee's mod)
	// Also check that the chosen TrueType font is available
	if( function_exists( 'imagettftext' ) && file_exists( $cas_font ) )
	{
		$cas_angle = 4; // Degrees to tilt the text
		$cas_offset = 15; // Pixels to offset the text from the border

		// Use png is available, since it produces clearer text images
		if( function_exists( 'imagepng' ) && $cas_UsePngNotJpeg )
		{
			imagettftext( $cas_image, 28, $cas_angle, $cas_offset, $cas_imgheight - $cas_offset, $cas_fontcolor, $cas_font, $cas_antispam );
			header( "Content-type: image/png" );
			imagepng( $cas_image );
		} else {
			imagettftext( $cas_image, 28, $cas_angle, $cas_offset, $cas_imgheight - $cas_offset, $cas_fontcolor, $cas_font, $cas_antispam );
			header( "Content-type: image/jpeg" );
			imagejpeg( $cas_image );
		}
	} else {
		$cas_fontsize = 5; // 1, 2, 3, 4 or 5 (higher numbers correspond to larger font sizes)
		$tmp_len = strlen( $cas_antispam );
		for( $tmp_count = 0; $tmp_count < $tmp_len; $tmp_count++ )
		{
		   $tmp_xpos = $tmp_count * imagefontwidth( $cas_fontsize ) + 20;
		   $tmp_ypos = 10;
		   imagechar( $cas_image, $cas_fontsize, $tmp_xpos, $tmp_ypos, $cas_antispam, $cas_fontcolor );
		   $cas_antispam = substr( $cas_antispam, 1);   
		}
		header("Content-Type: image/gif");
		imagegif( $cas_image );
	} // end if
	imagedestroy( $cas_image );

} else {

	// Determine the url to this script
	$tmp1 = strpos( __FILE__, ABSPATH );
	$tmp2 = strlen( ABSPATH);
	$cas_myurl = get_settings('siteurl') . "/" . substr( __FILE__, $tmp1 + $tmp2 );

	class PeterAntiSpam
	{
		function PeterAntiSpam()
		{
			global $cas_manualinsert;
			add_action( 'secure_image', array( &$this, 'comment_form' ) );    // add image and input field to comment form
			if (! $cas_manualinsert ) {
				add_action( 'comment_form', array( &$this, 'comment_form' ) );    // add image and input field to comment form
			}
			add_action( 'comment_post', array( &$this, 'comment_post' ) );    // add post comment post security code check
		}

		function comment_form()
		{
			global $cas_forcereg, $user_ID, $cas_textcount, $cas_myurl, $cas_imgheight, $cas_imgwidth, $cas_limitcolor, $cas_manualinsert, $cas_borderclr;
			// If the user is logged in, don't prompt for code
			if( ! $cas_forcereg && isset( $user_ID ) )
			{
				return( $post_ID );
			}

			// Pick a random number
			$cas_antiselect = rand( 1, $cas_textcount ); // 0 is for invalid, so don't select it
			echo( "\t\t\t".'<div style="display:block;" id="secureimgdiv">'."\n\t\t\t\t" );
			echo( '<p><label for="securitycode">Anti-spam word: (Required)</label><span style="color:#FF0000;">*</span><br />'."\n\t\t\t\t" );
			echo( "<small>To prove you're a person (not a spam script), type the security word shown in the picture.</small><br />\n\t\t\t\t" );
			echo( '<input type="text" name="securitycode" id="securitycode" size="30" tabindex="4" />'."\n\t\t\t\t" );
			echo( '<input type="hidden" name="matchthis" value="' . $cas_antiselect . "\" />\n\t\t\t\t" );
			echo( '<img src="' . $cas_myurl . '?antiselect=' . $cas_antiselect . "\"\n\t\t\t\t" );
			echo( 'alt="Anti-Spam Image" border="1" align="top" height="' . $cas_imgheight . '" width="' . $cas_imgwidth . "\"\n\t\t\t\t" );
			echo( 'style="border:thin solid ' . $cas_borderclr . ';' );
			echo( 'height:' . $cas_imgheight .';valign:top;width:' . $cas_imgwidth . ";\" /></p>\n\t\t\t" );
			echo( "</div>\n\t\t\t" );
if ( ! $cas_manualinsert ) {
			echo"<script language='JavaScript' type='text/javascript'>
			<!--
				var urlinput = document.getElementById(\"url\");
				var submitp = urlinput.parentNode;
				var substitution2 = document.getElementById(\"secureimgdiv\");
				submitp.appendChild(substitution2, urlinput);
			// -->
			</script>\n";
}
			return( $post_ID );
		}

		function comment_post( $post_ID )
		{
			global $_POST, $cas_text, $cas_textcount, $cas_forcereg, $user_ID, $comment_type, $cas_allowtrack, $wpdb;
			// Validate the form input values
			if( isset( $_POST['securitycode'] ) )
			{
				$securitycode = substr( strval( $_POST['securitycode'] ), 0, 50 );
			} else {
				$securitycode = '';
			}
			if( isset( $_POST['matchthis'] ) )
			{
				$matchnum = intval( $_POST['matchthis'] );
			} else {
				$matchnum = 0;
			}
			if( $matchnum < 1 || $matchnum > $cas_textcount ) $matchnum = 0;
			$matchthis = $cas_text[ $matchnum ];

			// If the user is not logged in check the security code
			if( $cas_forcereg || ! isset( $user_ID ) )
			{
				$istrackping = $wpdb->get_var("SELECT comment_type FROM {$wpdb->comments} WHERE comment_ID = {$post_ID}");
				if ( $istrackping == '' || ! $cas_allowtrack )
				{
					if ( $securitycode == '' )
					{
						$this->comment_rollback( $post_ID ); //roll back
						die( __('Error: Please enter the anti-spam word.') );
					}
					if ( strtolower( $matchthis ) != strtolower( $securitycode ) )
					{
						$this->comment_rollback($post_ID); //roll back
						die( __('Error: Please enter the correct anti-spam word. Press the back button and try again.') );
					} else {
						unset( $matchthis );
					}
				}
			}
			return( $post_ID );
		}

		function comment_rollback( $post_ID )
		{
			global $wpdb, $wp_version;
			$pid = $wpdb->get_var( "SELECT comment_post_ID FROM {$wpdb->comments} WHERE comment_ID = {$post_ID}" );
			$wpdb->query( "DELETE FROM {$wpdb->comments} WHERE comment_ID = {$post_ID}" ); //roll back
			if( $wp_version >= '2.0' )
			{
				$count = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->comments} WHERE comment_post_ID = {$pid} AND comment_approved = '1'" );
				$wpdb->query( "UPDATE {$wpdb->posts} SET comment_count = {$count} WHERE ID = {$pid}" );
			}
		}
	}
	new PeterAntiSpam();
}
?>